'' =================================================================================================
''
''   File....... jm_rfid_ez.spin
''   Purpose.... Simple RFID reader module
''               -- RFID reader is tied low (reader always active)
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 07 NOV 2015
''
'' =================================================================================================


con { fixed io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28


obj

  io  : "jm_io_basic"                                            '   basic io control
  sin : "jm_fullduplexserial"                                    ' * serial input for reader

' * Uses cog when loaded


var

  long  cog                                                      ' cog running serial io

  byte  tagbuf[11]                                               ' tag buffer 


pub null

  ' This is not a top-level object


pub start(rx)

'' Setup serial for RFID

  stop                                                           ' stop serial cog if running

  io.input(rx)                                                   ' release rx pin in this cog
  
  cog := sin.start(rx, rx, %1100, 2400)                          ' start serial cog (1w, open-drain)

  return cog


pub stop

'' Stops serial cog if running

  if (cog)
    cogstop(cog - 1)
    cog := 0                                                     ' mark stopped

  bytefill(@tagbuf, 0, 11)                                       ' clear tag buffer


pub check | c, idx

'' Check status of RFID system
'' -- returns true if RFID tag

  c := sin.rxcheck                                               ' check for serial input          
  if (c < 0)
    return false

  repeat
    if (c == $0A)                                                ' correct header?          
      bytefill(@tagbuf, 0, 11)                                   ' yes, clear buffer          
      repeat idx from 0 to 9                                     ' get 10 new tag bytes     
        c := sin.rxtime(5)                                                              
        if (c => 0)                                                                     
          tagbuf[idx] := c
        else                                                     ' broken tag comms?
          return false
      clear
      return true
                                                                      
    else
      c := sin.rxtime(5)                                         ' get another character from buffer
      if (c < 0)                                                 ' if timeout
        return false                                             '  no tag


pub clear

'' Clear serial input buffer
'' -- to remove duplicate tags

  sin.rxflush

  
pub address

'' Returns address of tagbuf
'' -- treated as 10-byte z-string

  return @tagbuf


pub tag_index(p_tags, tmax) | idx 

'' Returns index of current tag from list of tags
'' -- tags list must be stored as z-strings (10 ASCII bytes + 0)

  idx := 0

  repeat while (idx =< tmax)                                     ' iterate through list
    if (strcomp(@tagbuf, p_tags))                                ' if match
      return idx                                                 ' return index
    else
      ++idx                                                      ' next index
      p_tags += 11                                               ' skip over tag

  return -1                                                      ' not found in list


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen 

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}